#--
# Copyright (C) 2021 - Octopus Lab SAS
# All rights are described by the GPLv3 license present in the file /LICENSE available in the
# original repository of this file or [here](https://www.gnu.org/licenses/gpl-3.0.fr.html).
#++
# frozen_string_literal: true

require 'open-uri'
require 'json'

module GouvCalendarCompilator
  # This is the Data Compilator Object for the Gouv Data Compilator gem
  # It manages the compilation of different data sources and compile them into
  # a new, properly formatted one
  class DataCompilator
    # Initializer for Data Compiler class.
    def initialize
      @sh_data_compiler = ::GouvCalendarCompilator::France::SchoolHolidaysDataCompilator.new
      @ndo_data_compiler = ::GouvCalendarCompilator::France::NationalDaysOffDataCompilator.new
    end

    # This is the method you want to call to compile the sources into a proper dataset.
    # It returns a ruby Array of Hashes
    def compile
      sh_calendar_data = @sh_data_compiler.compile
      ndo_and_sh_data = self.class.sort_calendar_data(@ndo_data_compiler.compile(sh_calendar_data))
      self.class.fill_operating_days(ndo_and_sh_data)
    end

    # Sorts calendar data by date
    def self.sort_calendar_data(calendar)
      calendar.each_key do |zone_name|
        calendar[zone_name].sort_by! { |rec| ::Date.parse(rec[:start]) }
      end
      calendar
    end

    # Fill in the gaps in school holidays + national days off calendar with operatiog periods
    def self.fill_operating_days(calendar)
      calendar = sort_calendar_data(calendar)
      calendar.each_key do |zone|
        new_entries = []
        calendar[zone].each_with_index do |vacation_period, index|
          current_end_date = ::Date.parse(vacation_period[:end])
          if calendar[zone][index + 1].nil?
            if (current_end_date...::GouvCalendarCompilator::DATETIME_END).count > 1
              new_entries.push(
                {
                  start: current_end_date.to_date.to_s,
                  end: (::GouvCalendarCompilator::DATETIME_END + 1).to_date.to_s,
                  coefficient: ::GouvCalendarCompilator::OPERATING_DAY
                }
              )
            end
          else
            next_start_date = ::Date.parse(calendar[zone][index + 1][:start])

            if (current_end_date...next_start_date).count >= 1
              new_entries.push(
                {
                  start: current_end_date.to_date.to_s,
                  end: next_start_date.to_date.to_s,
                  coefficient: ::GouvCalendarCompilator::OPERATING_DAY
                }
              )
            end
          end
        end
        calendar[zone] += new_entries
      end
      sort_calendar_data(calendar)
    end
  end
end
